@@ -3,17 +3,9 @@ from __future__ import absolute_import, print_function
 import cython
 from .. import __version__
 
-import os
-import shutil
-import hashlib
-import subprocess
 import collections
-import re, sys, time
+import re, os, sys, time
 from glob import iglob
-from io import open as io_open
-from os.path import relpath as _relpath
-from distutils.extension import Extension
-from distutils.util import strtobool
 
 try:
     import gzip
@@ -22,13 +14,36 @@ try:
 except ImportError:
     gzip_open = open
     gzip_ext = ''
+import shutil
+import subprocess
 
 try:
-    import pythran
-    import pythran.config
-    PythranAvailable = True
-except:
-    PythranAvailable = False
+    import hashlib
+except ImportError:
+    import md5 as hashlib
+
+try:
+    from io import open as io_open
+except ImportError:
+    from codecs import open as io_open
+
+try:
+    from os.path import relpath as _relpath
+except ImportError:
+    # Py<2.6
+    def _relpath(path, start=os.path.curdir):
+        if not path:
+            raise ValueError("no path specified")
+        start_list = os.path.abspath(start).split(os.path.sep)
+        path_list = os.path.abspath(path).split(os.path.sep)
+        i = len(os.path.commonprefix([start_list, path_list]))
+        rel_list = [os.path.pardir] * (len(start_list)-i) + path_list[i:]
+        if not rel_list:
+            return os.path.curdir
+        return os.path.join(*rel_list)
+
+
+from distutils.extension import Extension
 
 from .. import Utils
 from ..Utils import (cached_function, cached_method, path_exists,
@@ -54,15 +69,6 @@ else:
     basestring = str
 
 
-def _make_relative(file_paths, base=None):
-    if not base:
-        base = os.getcwd()
-    if base[-1] != os.path.sep:
-        base += os.path.sep
-    return [_relpath(path, base) if path.startswith(base) else path
-            for path in file_paths]
-
-
 def extended_iglob(pattern):
     if '{' in pattern:
         m = re.match('(.*){([^}]+)}(.*)', pattern)
@@ -105,8 +111,7 @@ def nonempty(it, error_msg="expected non-empty iterator"):
 @cached_function
 def file_hash(filename):
     path = os.path.normpath(filename.encode("UTF-8"))
-    prefix = (str(len(path)) + ":").encode("UTF-8")
-    m = hashlib.md5(prefix)
+    m = hashlib.md5(str(len(path)) + ":")
     m.update(path)
     f = open(filename, 'rb')
     try:
@@ -151,7 +156,6 @@ def parse_list(s):
 
 transitive_str = object()
 transitive_list = object()
-bool_or = object()
 
 distutils_settings = {
     'name':                 str,
@@ -168,31 +172,9 @@ distutils_settings = {
     'export_symbols':       list,
     'depends':              transitive_list,
     'language':             transitive_str,
-    'np_pythran':           bool_or
 }
 
 
-def update_pythran_extension(ext):
-    if not PythranAvailable:
-        raise RuntimeError("You first need to install Pythran to use the np_pythran directive.")
-    pythran_ext = pythran.config.make_extension()
-    ext.include_dirs.extend(pythran_ext['include_dirs'])
-    ext.extra_compile_args.extend(pythran_ext['extra_compile_args'])
-    ext.extra_link_args.extend(pythran_ext['extra_link_args'])
-    ext.define_macros.extend(pythran_ext['define_macros'])
-    ext.undef_macros.extend(pythran_ext['undef_macros'])
-    ext.library_dirs.extend(pythran_ext['library_dirs'])
-    ext.libraries.extend(pythran_ext['libraries'])
-    ext.language = 'c++'
-
-    # These options are not compatible with the way normal Cython extensions work
-    for bad_option in ["-fwhole-program", "-fvisibility=hidden"]:
-        try:
-            ext.extra_compile_args.remove(bad_option)
-        except ValueError:
-            pass
-
-
 @cython.locals(start=cython.Py_ssize_t, end=cython.Py_ssize_t)
 def line_iter(source):
     if isinstance(source, basestring):
@@ -221,23 +203,19 @@ class DistutilsInfo(object):
                 if line[0] != '#':
                     break
                 line = line[1:].lstrip()
-                kind = next((k for k in ("distutils:","cython:") if line.startswith(k)), None)
-                if not kind is None:
-                    key, _, value = [s.strip() for s in line[len(kind):].partition('=')]
-                    type = distutils_settings.get(key, None)
-                    if line.startswith("cython:") and type is None: continue
+                if line[:10] == 'distutils:':
+                    key, _, value = [s.strip() for s in line[10:].partition('=')]
+                    type = distutils_settings[key]
                     if type in (list, transitive_list):
                         value = parse_list(value)
                         if key == 'define_macros':
                             value = [tuple(macro.split('=', 1))
                                      if '=' in macro else (macro, None)
                                      for macro in value]
-                    if type is bool_or:
-                        value = strtobool(value)
                     self.values[key] = value
         elif exn is not None:
             for key in distutils_settings:
-                if key in ('name', 'sources','np_pythran'):
+                if key in ('name', 'sources'):
                     continue
                 value = getattr(exn, key, None)
                 if value:
@@ -259,8 +237,6 @@ class DistutilsInfo(object):
                             all.append(v)
                     value = all
                 self.values[key] = value
-            elif type is bool_or:
-                self.values[key] = self.values.get(key, False) | value
         return self
 
     def subs(self, aliases):
@@ -398,30 +374,14 @@ def normalize_existing(base_path, rel_paths):
 
 @cached_function
 def normalize_existing0(base_dir, rel_paths):
-    """
-    Given some base directory ``base_dir`` and a list of path names
-    ``rel_paths``, normalize each relative path name ``rel`` by
-    replacing it by ``os.path.join(base, rel)`` if that file exists.
-
-    Return a couple ``(normalized, needed_base)`` where ``normalized``
-    if the list of normalized file names and ``needed_base`` is
-    ``base_dir`` if we actually needed ``base_dir``. If no paths were
-    changed (for example, if all paths were already absolute), then
-    ``needed_base`` is ``None``.
-    """
     normalized = []
-    needed_base = None
     for rel in rel_paths:
-        if os.path.isabs(rel):
-            normalized.append(rel)
-            continue
         path = join_path(base_dir, rel)
         if path_exists(path):
             normalized.append(os.path.normpath(path))
-            needed_base = base_dir
         else:
             normalized.append(rel)
-    return (normalized, needed_base)
+    return normalized
 
 
 def resolve_depends(depends, include_dirs):
@@ -522,25 +482,20 @@ class DependencyTree(object):
         return all
 
     @cached_method
-    def cimports_externs_incdirs(self, filename):
+    def cimports_and_externs(self, filename):
         # This is really ugly. Nested cimports are resolved with respect to the
         # includer, but includes are resolved with respect to the includee.
         cimports, includes, externs = self.parse_dependencies(filename)[:3]
         cimports = set(cimports)
         externs = set(externs)
-        incdirs = set()
         for include in self.included_files(filename):
-            included_cimports, included_externs, included_incdirs = self.cimports_externs_incdirs(include)
+            included_cimports, included_externs = self.cimports_and_externs(include)
             cimports.update(included_cimports)
             externs.update(included_externs)
-            incdirs.update(included_incdirs)
-        externs, incdir = normalize_existing(filename, externs)
-        if incdir:
-            incdirs.add(incdir)
-        return tuple(cimports), externs, incdirs
+        return tuple(cimports), normalize_existing(filename, externs)
 
     def cimports(self, filename):
-        return self.cimports_externs_incdirs(filename)[0]
+        return self.cimports_and_externs(filename)[0]
 
     def package(self, filename):
         return package(filename)
@@ -610,37 +565,25 @@ class DependencyTree(object):
 
     def transitive_fingerprint(self, filename, extra=None):
         try:
-            m = hashlib.md5(__version__.encode('UTF-8'))
-            m.update(file_hash(filename).encode('UTF-8'))
+            m = hashlib.md5(__version__)
+            m.update(file_hash(filename))
             for x in sorted(self.all_dependencies(filename)):
                 if os.path.splitext(x)[1] not in ('.c', '.cpp', '.h'):
-                    m.update(file_hash(x).encode('UTF-8'))
+                    m.update(file_hash(x))
             if extra is not None:
-                m.update(str(extra).encode('UTF-8'))
+                m.update(str(extra))
             return m.hexdigest()
         except IOError:
             return None
 
     def distutils_info0(self, filename):
         info = self.parse_dependencies(filename)[3]
-        kwds = info.values
-        cimports, externs, incdirs = self.cimports_externs_incdirs(filename)
-        basedir = os.getcwd()
-        # Add dependencies on "cdef extern from ..." files
+        externs = self.cimports_and_externs(filename)[1]
         if externs:
-            externs = _make_relative(externs, basedir)
-            if 'depends' in kwds:
-                kwds['depends'] = list(set(kwds['depends']).union(externs))
+            if 'depends' in info.values:
+                info.values['depends'] = list(set(info.values['depends']).union(externs))
             else:
-                kwds['depends'] = list(externs)
-        # Add include_dirs to ensure that the C compiler will find the
-        # "cdef extern from ..." files
-        if incdirs:
-            include_dirs = list(kwds.get('include_dirs', []))
-            for inc in _make_relative(incdirs, basedir):
-                if inc not in include_dirs:
-                    include_dirs.append(inc)
-            kwds['include_dirs'] = include_dirs
+                info.values['depends'] = list(externs)
         return info
 
     def distutils_info(self, filename, aliases=None, base=None):
@@ -693,20 +636,6 @@ def create_dependency_tree(ctx=None, quiet=False):
     return _dep_tree
 
 
-# If this changes, change also docs/src/reference/compilation.rst
-# which mentions this function
-def default_create_extension(template, kwds):
-    if 'depends' in kwds:
-        include_dirs = kwds.get('include_dirs', []) + ["."]
-        depends = resolve_depends(kwds['depends'], include_dirs)
-        kwds['depends'] = sorted(set(depends + template.depends))
-
-    t = template.__class__
-    ext = t(**kwds)
-    metadata = dict(distutils=kwds, module_name=kwds['name'])
-    return (ext, metadata)
-
-
 # This may be useful for advanced users?
 def create_extension_list(patterns, exclude=None, ctx=None, aliases=None, quiet=False, language=None,
                           exclude_failures=False):
@@ -739,26 +668,23 @@ def create_extension_list(patterns, exclude=None, ctx=None, aliases=None, quiet=
         Extension_distutils = Extension
         class Extension_setuptools(Extension): pass
 
-    # if no create_extension() function is defined, use a simple
-    # default function.
-    create_extension = ctx.options.create_extension or default_create_extension
-
     for pattern in patterns:
         if isinstance(pattern, str):
             filepattern = pattern
-            template = Extension(pattern, [])  # Fake Extension without sources
+            template = None
             name = '*'
             base = None
+            exn_type = Extension
             ext_language = language
         elif isinstance(pattern, (Extension_distutils, Extension_setuptools)):
             cython_sources = [s for s in pattern.sources
                               if os.path.splitext(s)[1] in ('.py', '.pyx')]
             if cython_sources:
-                filepattern = cython_sources[0]
-                if len(cython_sources) > 1:
-                    print("Warning: Multiple cython sources found for extension '%s': %s\n"
-                          "See http://cython.readthedocs.io/en/latest/src/userguide/sharing_declarations.html "
-                          "for sharing declarations among Cython files." % (pattern.name, cython_sources))
+              filepattern = cython_sources[0]
+              if len(cython_sources) > 1:
+                print("Warning: Multiple cython sources found for extension '%s': %s\n"
+                "See http://cython.readthedocs.io/en/latest/src/userguide/sharing_declarations.html "
+                "for sharing declarations among Cython files." % (pattern.name, cython_sources))
             else:
                 # ignore non-cython modules
                 module_list.append(pattern)
@@ -766,6 +692,7 @@ def create_extension_list(patterns, exclude=None, ctx=None, aliases=None, quiet=
             template = pattern
             name = template.name
             base = DistutilsInfo(exn=template)
+            exn_type = template.__class__
             ext_language = None  # do not override whatever the Extension says
         else:
             msg = str("pattern is not of type str nor subclass of Extension (%s)"
@@ -777,16 +704,16 @@ def create_extension_list(patterns, exclude=None, ctx=None, aliases=None, quiet=
         for file in nonempty(sorted(extended_iglob(filepattern)), "'%s' doesn't match any files" % filepattern):
             if os.path.abspath(file) in to_exclude:
                 continue
+            pkg = deps.package(file)
             module_name = deps.fully_qualified_name(file)
             if '*' in name:
                 if module_name in explicit_modules:
                     continue
-            elif name:
+            elif name != module_name:
+                print("Warning: Extension name '%s' does not match fully qualified name '%s' of '%s'" % (
+                    name, module_name, file))
                 module_name = name
 
-            if module_name == 'cython':
-                raise ValueError('cython is a special module, cannot be used as a module name')
-
             if module_name not in seen:
                 try:
                     kwds = deps.distutils_info(file, aliases, base).values
@@ -799,41 +726,42 @@ def create_extension_list(patterns, exclude=None, ctx=None, aliases=None, quiet=
                         if key not in kwds:
                             kwds[key] = value
 
-                kwds['name'] = module_name
-
-                sources = [file] + [m for m in template.sources if m != filepattern]
+                sources = [file]
+                if template is not None:
+                    sources += [m for m in template.sources if m != filepattern]
                 if 'sources' in kwds:
                     # allow users to add .c files etc.
                     for source in kwds['sources']:
                         source = encode_filename_in_py2(source)
                         if source not in sources:
                             sources.append(source)
-                kwds['sources'] = sources
+                    extra_sources = kwds['sources']
+                    del kwds['sources']
+                else:
+                    extra_sources = None
+                if 'depends' in kwds:
+                    depends = resolve_depends(kwds['depends'], (kwds.get('include_dirs') or []) + ["."])
+                    if template is not None:
+                        # Always include everything from the template.
+                        depends = set(template.depends).union(depends)
+                    # Sort depends to make the metadata dump in the
+                    # Cython-generated C code predictable.
+                    kwds['depends'] = sorted(depends)
 
                 if ext_language and 'language' not in kwds:
                     kwds['language'] = ext_language
 
-                np_pythran = kwds.pop('np_pythran', False)
-
-                # Create the new extension
-                m, metadata = create_extension(template, kwds)
-                m.np_pythran = np_pythran or getattr(m, 'np_pythran', False)
-                if m.np_pythran:
-                    update_pythran_extension(m)
-                module_list.append(m)
-
-                # Store metadata (this will be written as JSON in the
-                # generated C file but otherwise has no purpose)
-                module_metadata[module_name] = metadata
-
+                module_list.append(exn_type(
+                        name=module_name,
+                        sources=sources,
+                        **kwds))
+                if extra_sources:
+                    kwds['sources'] = extra_sources
+                module_metadata[module_name] = {'distutils': kwds, 'module_name': module_name}
+                m = module_list[-1]
                 if file not in m.sources:
-                    # Old setuptools unconditionally replaces .pyx with .c/.cpp
-                    target_file = os.path.splitext(file)[0] + ('.cpp' if m.language == 'c++' else '.c')
-                    try:
-                        m.sources.remove(target_file)
-                    except ValueError:
-                        # never seen this in the wild, but probably better to warn about this unexpected case
-                        print("Warning: Cython source file not found in sources list, adding %s" % file)
+                    # Old setuptools unconditionally replaces .pyx with .c
+                    m.sources.remove(file.rsplit('.')[0] + '.c')
                     m.sources.insert(0, file)
                 seen.add(name)
     return module_list, module_metadata
@@ -876,13 +804,6 @@ def cythonize(module_list, exclude=None, nthreads=0, aliases=None, quiet=False,
         if options.get('cache'):
             raise NotImplementedError("common_utility_include_dir does not yet work with caching")
         safe_makedirs(options['common_utility_include_dir'])
-
-    pythran_options = None
-    if PythranAvailable:
-        pythran_options = CompilationOptions(**options)
-        pythran_options.cplus = True
-        pythran_options.np_pythran = True
-
     c_options = CompilationOptions(**options)
     cpp_options = CompilationOptions(**options); cpp_options.cplus = True
     ctx = c_options.create_context()
@@ -898,41 +819,27 @@ def cythonize(module_list, exclude=None, nthreads=0, aliases=None, quiet=False,
     deps = create_dependency_tree(ctx, quiet=quiet)
     build_dir = getattr(options, 'build_dir', None)
 
-    def copy_to_build_dir(filepath, root=os.getcwd()):
-        filepath_abs = os.path.abspath(filepath)
-        if os.path.isabs(filepath):
-            filepath = filepath_abs
-        if filepath_abs.startswith(root):
-            # distutil extension depends are relative to cwd
-            mod_dir = join_path(build_dir,
-                                os.path.dirname(_relpath(filepath, root)))
-            copy_once_if_newer(filepath_abs, mod_dir)
-
-    modules_by_cfile = collections.defaultdict(list)
+    modules_by_cfile = {}
     to_compile = []
     for m in module_list:
         if build_dir:
+            root = os.getcwd()  # distutil extension depends are relative to cwd
+            def copy_to_build_dir(filepath, root=root):
+                filepath_abs = os.path.abspath(filepath)
+                if os.path.isabs(filepath):
+                    filepath = filepath_abs
+                if filepath_abs.startswith(root):
+                    mod_dir = join_path(build_dir,
+                            os.path.dirname(_relpath(filepath, root)))
+                    copy_once_if_newer(filepath_abs, mod_dir)
             for dep in m.depends:
                 copy_to_build_dir(dep)
 
-        cy_sources = [
-            source for source in m.sources
-            if os.path.splitext(source)[1] in ('.pyx', '.py')]
-        if len(cy_sources) == 1:
-            # normal "special" case: believe the Extension module name to allow user overrides
-            full_module_name = m.name
-        else:
-            # infer FQMN from source files
-            full_module_name = None
-
         new_sources = []
         for source in m.sources:
             base, ext = os.path.splitext(source)
             if ext in ('.pyx', '.py'):
-                if m.np_pythran:
-                    c_file = base + '.cpp'
-                    options = pythran_options
-                elif m.language == 'c++':
+                if m.language == 'c++':
                     c_file = base + '.cpp'
                     options = cpp_options
                 else:
@@ -959,7 +866,7 @@ def cythonize(module_list, exclude=None, nthreads=0, aliases=None, quiet=False,
                     dep_timestamp, dep = deps.newest_dependency(source)
                     priority = 2 - (dep in deps.immediate_dependencies(source))
                 if force or c_timestamp < dep_timestamp:
-                    if not quiet and not force:
+                    if not quiet:
                         if source == dep:
                             print("Compiling %s because it changed." % source)
                         else:
@@ -969,12 +876,13 @@ def cythonize(module_list, exclude=None, nthreads=0, aliases=None, quiet=False,
                         fingerprint = deps.transitive_fingerprint(source, extra)
                     else:
                         fingerprint = None
-                    to_compile.append((
-                        priority, source, c_file, fingerprint, quiet,
-                        options, not exclude_failures, module_metadata.get(m.name),
-                        full_module_name))
+                    to_compile.append((priority, source, c_file, fingerprint, quiet,
+                                       options, not exclude_failures, module_metadata.get(m.name)))
                 new_sources.append(c_file)
-                modules_by_cfile[c_file].append(m)
+                if c_file not in modules_by_cfile:
+                    modules_by_cfile[c_file] = [m]
+                else:
+                    modules_by_cfile[c_file].append(m)
             else:
                 new_sources.append(source)
                 if build_dir:
@@ -1090,15 +998,17 @@ else:
 
 # TODO: Share context? Issue: pyx processing leaks into pxd module
 @record_results
-def cythonize_one(pyx_file, c_file, fingerprint, quiet, options=None,
-                  raise_on_failure=True, embedded_metadata=None, full_module_name=None,
-                  progress=""):
-    from ..Compiler.Main import compile_single, default_options
+def cythonize_one(pyx_file, c_file, fingerprint, quiet, options=None, raise_on_failure=True, embedded_metadata=None, progress=""):
+    from ..Compiler.Main import compile, default_options
     from ..Compiler.Errors import CompileError, PyrexError
 
     if fingerprint:
         if not os.path.exists(options.cache):
-            safe_makedirs(options.cache)
+            try:
+                os.mkdir(options.cache)
+            except:
+                if not os.path.exists(options.cache):
+                    raise
         # Cython-generated c files are highly compressible.
         # (E.g. a compression ratio of about 10 for Sage).
         fingerprint_file = join_path(
@@ -1126,7 +1036,7 @@ def cythonize_one(pyx_file, c_file, fingerprint, quiet, options=None,
 
     any_failures = 0
     try:
-        result = compile_single(pyx_file, options, full_module_name=full_module_name)
+        result = compile([pyx_file], options)
         if result.num_errors > 0:
             any_failures = 1
     except (EnvironmentError, PyrexError) as e:
